#! /usr/bin/env python
import sys
import re
from Bio import SeqIO
from Bio.Seq import Seq

#A program that simplifies a Newbler (v2.3 and higher) transcriptome output so that only the longest isotig per isogroup is 
#written to the outfile (a .fasta file).  This will make BLASTing faster and more easy to summarize, since all the isotigs
#belonging to the same gene, ultimately, shouldn't be double-counted as independent BLAST hits.


usage="""
454_choose_longest_isotig.py

Usage: 

454choose_longest_isotig.py yourfastafile outfile
	
"""

#remind user of usage
if len(sys.argv)!=3:
	print usage	
	
else:

	fastafile = sys.argv[1]
	outfile = sys.argv[2]
	
	outfilesuffix="_only_longest_isotigs.fasta"
	outfile2suffix="_only_longest_isotigs_MISSING_ISOGROUPS.txt"
	outhandle=open(outfile + outfilesuffix, 'w')
	
	numseqs=0
	numisotigsoutput=0
	numisotigs=[]
	#parse the fasta input so you can iterate through it in order
	fastahandle=open(fastafile, 'r')

	regex = "gene=isogroup(\d+)  length=(\d+)"
	
	#initiate the parent_isogroup so that the first sequence registers as not the same
	parent_isogroup = 'XXX'
	seqlength=0
	
	longestisotig=Seq('')
	
	uniqueisotigs=[]
	#iterate through all the records in the fasta file
	
	for fasta_record in SeqIO.parse(fastahandle, 'fasta'):
		
		numseqs=numseqs+1
		#print 'NUMSEQ = ', numseqs, '\t', fasta_record.description
		
		regsearch=re.search(regex, fasta_record.description)
	
		#seqlength = regsearch.group(2)
		
		#IF THIS SEQ BELONGS TO SAME ISOGROUP AS PREVIOUS:	
		
		if regsearch.group(1) == parent_isogroup:
		
			#print 'SAME ISOGROUP'
			
			#if this sequence is longer than the other isotigs in the isogroup, update the longestcontig	
			#print regsearch.group(2), '>?', seqlength, max(regsearch.group(2), seqlength)
			
			if int(regsearch.group(2)) > int(seqlength):
				
				#print fasta_record.description, 'LONGER THAN PREVIOUS'
				longestisotig=fasta_record
				seqlength=regsearch.group(2)
			
			else:
				#print fasta_record.description, 'SHORTER THAN PREVIOUS'
				continue

		else:
		
			#print 'DIFFERENT ISOGROUP'
			
			#print longestisotig
			
			numisotigsoutput = numisotigsoutput+1 
			#numisotigs.append(numisotigsoutput)
			
			uniqueisotigs.append(longestisotig)
			
			parent_isogroup = regsearch.group(1)
			numisotigs.append(int(regsearch.group(1)))
		
			seqlength = regsearch.group(2)
			
			longestisotig = fasta_record
			
			continue
			
	#to get the last one, append one last time before outputting
	uniqueisotigs.append(longestisotig)	
	#print uniqueisotigs
	#have to exclude the first seq, which was emtpy just to initiate. 
	SeqIO.write(uniqueisotigs[1:], outhandle, "fasta")
	outhandle.close()
	print 'Number of sequences in original: ', numseqs, '\nNumber of Unique Isotigs (ie # Isogroups minus the inexplicably missing isogroups): ', numisotigsoutput,
	'DONE'
	
	outhandle2=open(outfile + outfile2suffix, 'w')
	
	outhandle2.write('Number of sequences in original: '+ str(numseqs) + '\nNumber of Unique Isotigs (ie # Isogroups minus the missing isogroups): '+ str(numisotigsoutput))
	
	outhandle2.write('\n\nHere are the isogroups that don\'t exist in '+ fastafile+ '/n(You can grep for the contigs in the 454AllContigs.fna file\n\n')
	for num in range(len(numisotigs)-1):
		if numisotigs[num+1] - 1 != numisotigs[num]:
			outhandle2.write(str(numisotigs[num]) + '-->' + str(numisotigs[num+1])+'\n')
		else:
			continue
			