#! /usr/bin/env python
import sys
import re
from Bio import SeqIO
from Bio.Blast import NCBIXML

#A program to calculate the hit length ratio, which is defined by the length of a BLAST hit divided
#by the length of the protein that it hits.  This ratio was defined and is explained well in O'Neil et al 2010 (PMID: 20478048)

#output has two columns: OHR, subjectlength
#OHR: length of query_blast_hit divided by length of subject (can be over 1 if there are insertions)
#subjectlength: length of the blast hit, to make a scatterplot as in O'Neil et al 2010 Figure 5a,b.

#If you want to see what the program is doing, remove the '#' signs 

usage="""
ortholog_hit_ratio.py

Usage: 

ortholog_hit_ratio.py blast_result_template num_blast_result_files[interger] outfile	
"""

#remind user of usage
if len(sys.argv)<3:
	print usage	

else:

	numfiles=int(sys.argv[2])
	blastfile=sys.argv[1]
	outfile=sys.argv[3]
	
	outhandle=open(outfile,'w')
	
#iterate through all of the blast files (assumes split blast inputs)
	for eachfile in range(numfiles+1):
	
		if eachfile == 0:
		
			continue
			
		else:
	
			blasthandle=open(blastfile + str(eachfile) + '.xml', 'r')
#			blast_records=NCBIXML.parse(blasthandle)
		
			for blast_record in NCBIXML.parse(blasthandle):
			
#figure out if there is a blast hit with a try
				try:
					blast_record.descriptions[0]!=''
			
#if there is a blast hit, get the start and stop (max, ignoring indels with the third "for loop")

#					print blast_record.query_id, ': ', blast_record.query
					for alignment in blast_record.alignments[0:1]:
			#			print alignment.title
						subject_length=int(alignment.length)
#						print alignment.title, ': ', subject_length
						for hsp in alignment.hsps[0:1]:
							topstart=hsp.sbjct_start
							topstop=hsp.sbjct_end
							querystart=hsp.query_start
							querystop=hsp.query_end
#							print querystart, querystop
						for hsp in alignment.hsps[1:]: #to account for frameshifts etc, take min and max subct_start and stop for all hsps
							topstart=min(topstart, hsp.sbjct_start)
							topstop=max(topstop, hsp.sbjct_end)
							querystart=min(querystart, hsp.query_start)
							querystop=max(querystop, hsp.query_end)
#							print 'UPDATED', querystart, querystop
#calculate ortholog_hit_ratio

					
					#NOTE: the blast results are given in nucleotides but the subject in amino acids so divide by 3!)
					ortholog_hit_ratio2=float(querystop-querystart)/(3*float(subject_length)) #this is length of query that hits the subject divided by the length of the full subject
					
					
					outhandle.write(str(ortholog_hit_ratio2)+'\t'+str(subject_length)+'\n')
		
					continue
					
#if there is no blast hit, just move on

				except IndexError:
				
#					print blast_record.query_id, ': NO HITS'
					
					continue
		
	
outhandle.close()

	
